import requests
from requests.adapters import HTTPAdapter, Retry
from RupineLib import db, timer, dataLoadJobHelper

class TaskSetUp:
    '''
    setting up basic variables (e.g. dagId or taskId), connections and helper classes.
    This setup method should be more or less static for most of the load jobs from data sources

    :param kwargs: for local usage it is kwargs dictionary in __main__; for Airflow usage it is from op_kwargs of Python Operator
    :param loopAttributes: if attributes from op_attributes which occur in data load jobs should be used in a loop, then provide the attribute number in a list.
    '''
    def __init__(self,kwargs:dict) -> None:
        ####### Getting args 
        # local: kwargs dictionary in __main__
        # Airflow: from op_kwargs of Python Operator
        source = self.__nvl(kwargs,'source')
        databaseConnection = self.__nvl(kwargs,'databaseConnection')
        self.dagId = self.__nvl(kwargs,'dagId')
        self.taskId = self.__nvl(kwargs,'taskId')

        ####### HTTP Request setup
        self.requestSession = None
        self.customizeRequestSession()

        ####### Database setup
        if source == 'local' and databaseConnection is not None:
            self.connection = db.connectURI(databaseConnection)
        elif source == 'airflow' and databaseConnection is not None:
            self.connection = db.connect(databaseConnection.login,databaseConnection.password,databaseConnection.host,databaseConnection.port,databaseConnection.schema)
        else:
            self.connection = None

        ####### Setup timer
        self.timing = timer.Timer()

        self.jobHelper = None

    def setUpDataLoadJobHelper(self,env,loopAttributes:list=[]):
        if self.dagId is not None and self.taskId is not None:
            ####### Setup data load job Helper
            dataLoadJob = db.SELECT_FUNCTION(self.connection,'op_prod','op_create_data_load_job',[self.dagId,self.taskId,env])
            self.jobHelper = dataLoadJobHelper.DataLoadJobHelper({
                'loopAttributes': loopAttributes
            })
            self.jobHelper.init(dataLoadJob)

    def customizeRequestSession(self,total=5,backoff_factor=1,status_forcelist=[ 502, 503, 504 ]):
        self.requestSession = requests.Session()
        retries = Retry(total=total, backoff_factor=backoff_factor, status_forcelist=status_forcelist)
        self.requestSession.mount('https://', HTTPAdapter(max_retries=retries))

    def __nvl(self,obj,key,default=None):
        if key not in obj:
            return default
        else:
            return obj[key]
